package org.bench4q.web.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.bench4q.share.models.agent.BehaviorBriefModel;
import org.bench4q.share.models.agent.BehaviorStatusCodeResultModel;
import org.bench4q.share.models.agent.ParameterModel;
import org.bench4q.share.models.agent.RunScenarioModel;
import org.bench4q.share.models.agent.scriptrecord.BatchModel;
import org.bench4q.share.models.agent.scriptrecord.BehaviorModel;
import org.bench4q.share.models.agent.scriptrecord.PageModel;
import org.bench4q.share.models.master.RunningScriptModel;
import org.bench4q.share.models.master.TestPlanScriptBriefResultModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorsBriefModel;
import org.bench4q.share.models.master.statistics.ScriptBriefResultModel;
import org.bench4q.web.exception.CustomGenericException;
import org.bench4q.web.model.ResultInfoModel;
import org.bench4q.web.model.ResultModel;
import org.bench4q.web.model.ScriptModel;
import org.bench4q.web.model.TestPlanRequestModel;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.SessionAttributes;

@Component
@SessionAttributes({ "accessToken", "testPlanTaskList" })
public class TestPlanService extends BaseService {

	/*
	 * private final String BASECALLER = "TestPlanService:";
	 * 
	 * private String getBaseUrl() { return baseUrl; }
	 */
	public TestPlanService() {
		// TODO Auto-generated constructor stub
	}

	public ResultInfoModel getScriptBriefStatusResultInfo() {
		String result = "briefStatus";

		String[] fields = new String[] { "averageResponseTime",
				"maxResponseTime", "minResponseTime", "averageElapsedTime",
				"successThroughputThisTime", "failThroughputThisTime",
				"failRateThisTime", "totalFailCountFromBegin",
				"responseTimeDeviationThisTime" };
		return getResultInfoModel(result, fields);
	}

	public ResultInfoModel getScriptBehaviorBriefResultInfo(
			RunScenarioModel runScenarioModel) {
		String behaviroResult = "behaviors";

		ResultInfoModel behaviorResultInfoModel = new ResultInfoModel();
		behaviorResultInfoModel
				.setChildResults(new ArrayList<ResultInfoModel>());
		behaviorResultInfoModel.setResult(behaviroResult);

		String[] fields = new String[] { "minResponseTime", "maxResponseTime",
				"totalResponseTimeThisTime" };
		for (String url : this.getScriptBehaviorUrl(runScenarioModel)) {
			ResultInfoModel urlResultInfoModel =

			getResultInfoModel(url, fields);
			behaviorResultInfoModel.getChildResults().add(urlResultInfoModel);
		}

		return behaviorResultInfoModel;
	}

	private List<String> getScriptPages(RunScenarioModel runScenarioModel) {
		try {
			List<String> scriptPages = new ArrayList<String>();
			for (int i = 1; i < scriptPages.size(); i++) {
				scriptPages.add("page_" + i);
			}
			return scriptPages;
		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	private List<String> getScriptBehaviorUrl(RunScenarioModel runScenarioModel) {

		List<BehaviorModel> behaviorModelsWithUrl = getBehaviorModelsHasUrl(runScenarioModel);
		if (behaviorModelsWithUrl == null)
			return null;
		List<String> urList = new ArrayList<String>();
		for (BehaviorModel behaviorModelWithUrl : behaviorModelsWithUrl) {
			String url = generateUrl(behaviorModelWithUrl.getParameters());
			urList.add(url);
		}
		return urList;
	}

	private String generateUrl(List<ParameterModel> parameterModels) {
		String url = new String("");
		int i = 0;
		for (ParameterModel parameterModel : parameterModels) {
			if (i == 1 && parameterModel.getValue() != null)
				url += "?" + parameterModel.getValue();
			if (i > 1 && parameterModel.getValue() != null)
				url += "&" + parameterModel.getValue();
			i++;
		}
		if (url.length() <= 0)
			return null;
		return url;
	}

	private List<BehaviorModel> getBehaviorModelsHasUrl(
			RunScenarioModel runScenarioModel) {
		try {

			List<BehaviorModel> behaviorModels = getBehaviorModels(runScenarioModel);
			if (behaviorModels == null)
				return null;
			List<BehaviorModel> behaviorModelsWithUrl = new ArrayList<BehaviorModel>();
			for (BehaviorModel behaviorModel : behaviorModels) {
				List<ParameterModel> parameterModels = new ArrayList<ParameterModel>();
				parameterModels.addAll(behaviorModel.getParameters());
				for (ParameterModel parameterModel : parameterModels) {
					if (parameterModel.getKey().equals("url"))
						behaviorModelsWithUrl.add(behaviorModel);
					break;
				}
			}

			return behaviorModelsWithUrl;
		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	private List<BehaviorModel> getBehaviorModels(
			RunScenarioModel runScenarioModel) {
		try {
			List<BatchModel> batchModels = new ArrayList<BatchModel>();
			for (PageModel pageModel : runScenarioModel.getPages())
				batchModels.addAll(pageModel.getBatches());
			List<BehaviorModel> behaviorModels = new ArrayList<BehaviorModel>();
			for (BatchModel batchModel : batchModels)
				behaviorModels.addAll(batchModel.getBehaviors());
			return behaviorModels;
		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	public ResultInfoModel getScriptPageBriefResultInfo(
			RunScenarioModel runScenarioModel) {

		ResultInfoModel pageInfoModel = new ResultInfoModel();
		pageInfoModel.setChildResults(new ArrayList<ResultInfoModel>());
		pageInfoModel.setResult("Pages");
		String[] fields = new String[] { "averageResponseTimeThisTime",
				"latestResponseTime", "throughputThisTime" };
		for (String page : getScriptPages(runScenarioModel)) {

			pageInfoModel.getChildResults().add(
					getResultInfoModel(page, fields));
		}
		return pageInfoModel;
	}

	public ResultInfoModel getResultInfoModel(String result, List<String> fields) {
		ResultInfoModel resultInfoModel = new ResultInfoModel();
		resultInfoModel.setResult(result);
		if (fields.size() <= 0)
			return resultInfoModel;

		resultInfoModel.setChildResults(new ArrayList<ResultInfoModel>());
		for (String field : fields) {
			ResultInfoModel scriptResultInfoModelChild = new ResultInfoModel();

			scriptResultInfoModelChild.setResult(field);
			resultInfoModel.getChildResults().add(scriptResultInfoModelChild);
		}
		return resultInfoModel;
	}

	public Map<String, Integer> getScriptIdAndNameMap(
			List<RunningScriptModel> runningScriptModels) {
		Map<String, Integer> map = new HashMap<String, Integer>();
		for (RunningScriptModel runningScriptModel : runningScriptModels) {
			map.put(runningScriptModel.getScriptName(),
					runningScriptModel.getScriptId());
		}
		return map;
	}

	public Map<String, Integer> getPageNameAndIdMap(
			List<RunningScriptModel> runningScriptModels) {
		try {
			Map<String, Integer> map = new HashMap<String, Integer>();
			for (RunningScriptModel runningScriptModel : runningScriptModels) {
				Map<String, Integer> pageMap = getPageNameAndIdMap(runningScriptModel
						.getScenarioModel());
				if (pageMap != null)
					map.putAll(pageMap);
			}
			return map;
		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	private Map<String, Integer> getPageNameAndIdMap(
			RunScenarioModel runScenarioModel) {

		List<PageModel> pageModels = runScenarioModel.getPages();
		try {
			Map<String, Integer> map = new HashMap<String, Integer>();
			for (int i = 0; i < pageModels.size(); i++) {
				map.put("page_" + i, i);
			}
			return map;
		} catch (NullPointerException e) {
			return null;
		}

	}

	public Map<String, Integer> getBehaviorNameAndIdMap(
			List<RunningScriptModel> runningScriptModels) {
		try {
			Map<String, Integer> map = new HashMap<String, Integer>();
			for (RunningScriptModel runningScriptModel : runningScriptModels) {
				Map<String, Integer> behaviorMap = getBehaviorNameAndIdMap(runningScriptModel
						.getScenarioModel());
				if (behaviorMap != null)
					map.putAll(behaviorMap);
			}
			return map;
		} catch (NullPointerException e) {
			return null;
		}
	}

	private Map<String, Integer> getBehaviorNameAndIdMap(
			RunScenarioModel runScenarioModel) {
		try {
			Map<String, Integer> map = new HashMap<String, Integer>();
			List<BehaviorModel> behaviorModels = this
					.getBehaviorModelsHasUrl(runScenarioModel);
			if (behaviorModels == null)
				return null;
			for (BehaviorModel behaviorModel : behaviorModels) {
				String url = this.generateUrl(behaviorModel.getParameters());
				if (url != null)
					map.put(url, behaviorModel.getId());
			}
			return map;

		} catch (NullPointerException e) {
			e.printStackTrace();
			return null;
		}
	}

	public List<ResultModel> extractResultModelList(
			TestPlanScriptBriefResultModel testPlanScriptBriefResultModel,
			String fieldName) throws CustomGenericException,
			NoSuchFieldException {
		if (testPlanScriptBriefResultModel.getScriptBriefResultModels() == null)
			throw new CustomGenericException(
					"5",
					"testPlanScriptBriefResultModel:scriptBriefResultModels  empty  data",
					"");
		List<ResultModel> resultModels = new ArrayList<ResultModel>();
		for (ScriptBriefResultModel scriptBriefResultModel : testPlanScriptBriefResultModel
				.getScriptBriefResultModels()) {

			List<String> fieldNames = new ArrayList<String>();
			fieldNames.add(fieldName);
			ResultModel resultModel = ChangeToResultModel.getResultModel(
					scriptBriefResultModel, fieldNames);
			resultModel.setFinished(scriptBriefResultModel.isFinished());
			resultModel.setTime(scriptBriefResultModel.getSamplingTime());
			resultModels.add(resultModel);
		}
		return resultModels;

	}

	public boolean isValidateTestRunModel(
			TestPlanRequestModel testPlanRequestModel) {

		if (testPlanRequestModel.getScriptList() == null
				|| testPlanRequestModel.getScriptList().size() == 0)
			return true;
		if (testPlanRequestModel.getTestPlanName() == null
				|| testPlanRequestModel.getTestPlanName().equals(""))
			testPlanRequestModel.setTestPlanName("default");
		for (ScriptModel scriptModel : testPlanRequestModel.getScriptList()) {
			if (isValidateScriptModel(scriptModel))
				return true;
		}

		return false;
	}

	private boolean isValidateScriptModel(ScriptModel scriptModel) {
		if (scriptModel.getId() == 0)
			return true;
		if (scriptModel.getLoad() == 0)
			return true;
		if (scriptModel.getExecuteRange() == 0)
			return true;
		return false;
	}

	public Map<String, Double> getUrlContentTypePercent(
			Map<String, Long> urlContentTypeCountMap, long totalCount) {
		if (urlContentTypeCountMap == null)
			return null;

		if (totalCount == 0)
			return null;
		Map<String, Double> urlContentTypePercentMap = new HashMap<String, Double>();
		Set<String> keys = urlContentTypeCountMap.keySet();
		Iterator<String> iterator = keys.iterator();
		while (iterator.hasNext()) {
			String key = iterator.next();
			double percent = (double) urlContentTypeCountMap.get(key)
					/ (double) totalCount * 100;
			urlContentTypePercentMap.put(key, new Double(percent));
		}
		return urlContentTypePercentMap;

	}

	public long getTotalCount(Map<String, Long> urlContentTypeCountMap) {

		Set<String> keys = urlContentTypeCountMap.keySet();
		Iterator<String> iterator = keys.iterator();
		long totalCount = 0;
		while (iterator.hasNext()) {
			totalCount += urlContentTypeCountMap.get(iterator.next());
		}
		return totalCount;
	}

	public List<BehaviorStatusCodeResultModel> getBehaviorStatusCodeResultModels(
			ScriptBehaviorsBriefModel scriptBehaviorsBriefModel) {
		List<BehaviorStatusCodeResultModel> behaviorStatusCodeResultModels = new ArrayList<BehaviorStatusCodeResultModel>();
		for (BehaviorBriefModel behaviorBriefModel : scriptBehaviorsBriefModel
				.getTestBehaviorsBriefModel().getBehaviorBriefModels()) {

			if (behaviorBriefModel.getDetailStatusCodeResultModels() != null)
				behaviorStatusCodeResultModels.addAll(behaviorBriefModel
						.getDetailStatusCodeResultModels());
		}
		return behaviorStatusCodeResultModels;

	}

}
